@@ -108,10 +108,15 @@
 	       vals)
      ,@(map (lambda (val)
 	      (if (pair? (cddr val))
-		  `(set! (setter ',(car val))
-			 (if (not (,(caddr val) ,(car val))) ; check initial value (already set)
-			     (error 'wrong-type "typed-let: initial value ~S is not ~S" ,(car val) ,(caddr val))
-			     ,(caddr val)))                  ; assume built-in type here
+		  (let ((init-args (if (pair? (caddr val)) ; setter: (lambda (s v [e]) ...)
+				       (if (eqv? (length (cadr (caddr val))) 3)
+					   (list (list 'quote (car val)) (car val) (list 'curlet))
+					   (list (list 'quote (car val)) (car val)))
+				       (list (car val))))) ; setter: integer? etc
+		    `(begin
+		       (unless (,(caddr val) ,@init-args) ; check initial value (already set)
+			 (error 'wrong-type "typed-let: '~S initial value ~S is not ~S" ',(car val) ,(car val) ,(caddr val)))
+		       (set! (setter ',(car val)) ,(caddr val))))
 		  (values)))
 	    vals)
      ,@body))
@@ -1349,13 +1354,6 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 
 ;;; ----------------
 
-(define-macro (catch* clauses . error)
-  (let builder ((lst clauses))
-    (if (null? lst)
-	(apply values error)
-	`(catch #t (lambda () ,(car lst)) (lambda args ,(builder (cdr lst)))))))
-
-
 (define* (subsequence obj (start 0) end)
   (let ((new-len (let ((len (length obj)))
 		   (- (min len (or end len)) start))))
@@ -1371,9 +1369,11 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
                (substring obj start)))
 
           ((not (pair? obj))
-           (catch* (((obj 'subsequence) obj start end)
-		    (subsequence (obj 'value) start end))
-		   #f))
+           (if (defined? 'value obj #t)
+	       (subsequence (obj 'value) start end)
+	       (if (defined? 'subsequence obj #t)
+		   ((obj 'subsequence) obj start end)
+		   #f)))
 
           ((not end)
 	   (list-tail obj start))
